unit SharedMemStream;

// Yves LEMAIRE alias tourlourou - 2006

interface

uses
  Classes;

type
    // dfinit un type de MemoryStream pointant sur une portion seulement de la mmoire d'un MemoryStream
    // intrt : passer cette zone  une procdure LoadFromStream d'un objet quelconque, sans duplication
  TSharedMemoryStream=class(TCustomMemoryStream)
  private
    // drapeau d'interdiction de la procdure SetPointer
    FSetFlag: Boolean;
  protected
    // surcharge de la mthode pour ne l'autoriser qu'au Create
    procedure SetPointer(Ptr: Pointer; Size: Longint);
  public
    // constructeur, qui fera pointer sur un MemoryStream-pre
    constructor Create(AMemoryStream: TMemoryStream; APos, ACount: LongInt);
    // redfinition pour interdire la modification de la zone mmoire du pre !
    function Write(const Buffer; Count: Longint): Longint; override;
  end;
  
implementation

constructor TSharedMemoryStream.Create(AMemoryStream: TMemoryStream; APos, ACount: LongInt);
var
  Ptr: Pointer;
begin
  if (AMemoryStream=nil)
  or (APos<0)
  or (ACount<=0)
  or (APos+ACount>AMemoryStream.Size)
  then raise EStreamError.Create('Paramtres errons !');
  inherited Create;
    // obtention de l'adresse du bloc mmoire du stream d'origine (pour Position=0)
  Ptr:=AMemoryStream.Memory;
    // ajout de l'offset de la zone  atteindre (pour Position=Apos)
  Ptr:=Pointer(Cardinal(Ptr)+APos);
    // attribution de l'origine de ce bloc mmoire et de sa taille
    // crant un stream virtuel pointant sur une portion seulement d'un MemoryStream
  SetPointer(Ptr, ACount);
end;

procedure TSharedMemoryStream.SetPointer(Ptr: Pointer; Size: Longint);
begin
    // test du drapeau qui interdit de modifier la zone pointe au Create
  if FSetFlag
  then raise EReadError.Create('Dsol, flux en lecture seule !')
  else inherited;
    // positionnement du drapeau
  FSetFlag:=True;
end;

function TSharedMemoryStream.Write(const Buffer; Count: Longint): Longint;
begin
    // leve d'une exception : le TSharedMemoryStream est Read-Only
  raise EReadError.Create('Dsol, flux en lecture seule !');
end;

end.
 